﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;


using System.Data;
using FirebirdSql.Data.FirebirdClient;
using System.IO;
using System.Windows;  // MessageBox


namespace Company.DBS_Klassen {
    public class Dbs {

        // Singleton
        private FbConnection _cnn = null;

        private static Dbs dbs = new Dbs();

        public static Dbs getInstance() {
            return dbs;
        }


        // holt die aktuelle Sequenznummer fuers Backup
        public static int getSequenzActualValue(String seqname) {
            String sql = "select gen_id( " + seqname + ",0)  FROM RDB$DATABASE;";
            using (DataTable dt1 = dbs.selectDBS(sql)) {
                foreach (System.Data.DataRow row in dt1.Rows) {
                    Object[] objs = row.ItemArray;
                    long index = (long)objs[0];
                    return (int)index;
                }
                return 0;
            }
        }

        // holt eine neue Sequenznummer 
        public static int getSequenz(String seqname) {
            String sql = "SELECT NEXT VALUE FOR " + seqname + " FROM RDB$DATABASE;";
            using (DataTable dt1 = dbs.selectDBS(sql)) {
                foreach (System.Data.DataRow row in dt1.Rows) {
                    Object[] objs = row.ItemArray;
                    long index = (long)objs[0];
                    return (int)index;
                }
                return 0;
            }
        }


        // setzt die Datenbankverbindung, Firebird-Format
        public void setDbs(string sDatabaseFile) {
            // Environment.GetFolderPath(Environment.SpecialFolder.Personal);
            string sPath;
            sPath = Environment.CurrentDirectory;
            //sPath = "C:\\Daten";
            if (!sPath.EndsWith("\\")) sPath += "\\";
            string sArg =
                "User=SYSDBA;" +
                "Password=masterkey;" +
                "Database=" + sDatabaseFile + ";" +
                "DataSource=localhost;" +
                "Port=3050;" +
                "Dialect=3;" +
                "Charset=NONE;" +
                "Role=;" +
                "Connection lifetime=15;" +
                "Pooling=true;" +
                "MinPoolSize=0;" +
                "MaxPoolSize=50;" +
                "Packet Size=8192;" +
                "ServerType=1";

            if (_cnn != null) {
                //if (_cnn.Close();  // pro Forma
            }
            _cnn = new FbConnection(sArg);
            if (_cnn == null)
                MessageBox.Show("Fehler beim Öffnen der Datenbank\r\n" +
                      sPath + sDatabaseFile, "Fehler");
            else
                _cnn.Open();
        }  // dbs



        //_dbs.createDatabase("C:\\Daten\\test1.fdb");
        //_dbs.createDatabase(@"C:\Daten\test1.fdb");
        public bool createDatabase(string filename) {
            if (File.Exists(filename)) {
                try {
                    File.Delete(filename);
                }
                catch (Exception) {
                    Basis.ErrorMsg("Die Datenbankdatei ist seitens des Betriebssystem noch offen\r\n" +
                        "Bitte das Programm schließen und erneut erzeugen", "Hinweis");
                    return false;
                }
            }
            string sArg1 =
                "User=SYSDBA;" +
                "Password=masterkey;" +
                "Database=" + filename + ";" +
                "DataSource=localhost;" +
                "Port=3050;" +
                "Dialect=3;" +
                "Charset=NONE;" +
                "Role=;" +
                "Connection lifetime=15;" +
                "Pooling=true;" +
                "MinPoolSize=0;" +
                "MaxPoolSize=50;" +
                "Packet Size=8192;" +
                "ServerType=1";

            string sArg =
                "User=SYSDBA;" +
                "Password=masterkey;" +
                "Database=" + filename + ";" +
                "DataSource=localhost;" +
                "Dialect=3;" +
                "Packet Size=8192;" +
                "ServerType=1";
            FbConnection.CreateDatabase(sArg);
            return true;
            // Alternative Parameter
            //FbConnection.CreateDatabase(connectionstring, pagesize, forcedwrite, overwrite
        }

        // schließt die Verbindung
        public void Close() {
            if (_cnn != null) {
                _cnn.Close();
                _cnn = null;
            }
        }



        // Methode für das Update, Delete, Drop, Alter Table
        public void modifyDBS(string sSQL) {
            FbDataAdapter da = new FbDataAdapter(sSQL, _cnn);
            try {
                {
                    DataTable dt = new DataTable("RELATIONS");
                    try {
                        da.Fill(dt);
                    }
                    finally {
                        if (dt != null) ((IDisposable)dt).Dispose();
                    }
                }
            }
            catch (Exception e) {
                MessageBox.Show("Fehlerhafte SQL-Anweisung\r\n" + e.Message + "\r\n" + sSQL, "SQL-Fehler", MessageBoxButton.OK, MessageBoxImage.Error);
                return;
            }
            finally {
                if (da != null) ((IDisposable)da).Dispose();
            }
        }

        // allgemeine Datenbankabfrage fuer SQL-Abfragen
        public DataTable selectDBS(string sSQL) {
            try {
                FbDataAdapter da = new FbDataAdapter(sSQL, _cnn);
                DataTable dt = new DataTable("RELATIONS");
                da.Fill(dt);
                return dt;
            }
            catch (Exception e) {
                MessageBox.Show("Fehlerhafte SQL-Anweisung\r\n" + e.Message + "\r\n" + sSQL, "SQL-Fehler", MessageBoxButton.OK, MessageBoxImage.Error);
                return null;
            }
        }



        // hier kann man mehrere Tabellen erhalten, wird nicht benoetigt
        public DataSet getDBSDataSet(string sSQL, string sTable) {
            FbDataAdapter da = new FbDataAdapter(sSQL, _cnn);
            //FbCommand command = new FbCommand(sSQL, _cnn);
            DataSet ds = new DataSet();
            //DataTable dt = new DataTable("RELATIONS");

            int n = ds.Tables.Count;
            da.Fill(ds, sTable);  // z. B. "Employee", wichtig da es mehrere gibt
            n = ds.Tables.Count;  // Anzahl der internen Tabellen
                                  //da.Dispose();
            return ds;
        }

        // Ruft die internen Strukturen der Datenbanken ab, benoetigt bei der DB-Konsole
        public DataTable getTables() {
            string[] filter = new string[4];
            filter[0] = "dbo"; //  null;
            filter[1] = null;
            filter[2] = null;
            filter[3] = null;
            DataTable dt;
            dt = GetSchema("Tables", filter);
            return dt;
        }

        // Ruft die internen Strukturen der Datenbanken ab, benoetigt bei der DB-Konsole
        public DataTable GetSchema(string sItem, string[] filter) {
            return _cnn.GetSchema(sItem, filter);
        }

        // Ruft die internen Strukturen der Datenbanken ab, benoetigt bei der DB-Konsole
        public DataTable getDataTyps(string sTable) {
            string[] filter = new string[4];  // Schränkt das Ergebnis ein
            filter[0] = null; // Katalog  TABLE_CATALOG     z. B. dbo
            filter[1] = null; // Besitzer  TABLE_SCHEMA
            filter[2] = sTable;  // Tabellenname
            filter[3] = null;  // TABLE_TYPE  (System oder 
            DataTable dt;
            dt = new DataTable("RELATIONS");
            // dt = _cnn.GetSchema("Columns");  // liefert alle Spalten
            dt = GetSchema("Columns", filter);
            return dt;
        }
    }
}
