﻿
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Input;
using System.Windows.Media;

using System.Windows.Controls.Ribbon;
using Microsoft.Win32;
using Company.DBS_Klassen;
using System.IO;
using Company.ViewModels;
using Company.Models;
using Company.Views;
using Company.ViewsDialog;
using Company.Dialogs;

// Referenz hinzufügen
// Assemblies  suchen nach Ribbon 

namespace Company {
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : RibbonWindow {

        private String dbtable = "MainWindow";

        String databasefile = null;
        String registryfilename = "";

        List<ModelDept> dept_Items = null;



        public MainWindow() {
            InitializeComponent();
        }

        // Neu: holen der letzten Datenbank: 
        private void Window_Loaded(Object sender, RoutedEventArgs e) {
            setGUI();
            registryfilename = Basis.getFilename(registryfilename);
            if (!registryfilename.Equals("") && File.Exists(registryfilename)) {
                tLastdatabasefile.Text = registryfilename;
                tLastdatabasefile.ToolTip = registryfilename;
            }
            else {
                registryfilename = "";
                tLastdatabasefile.IsEnabled = false;
                bnOpenLastDatenbase.IsEnabled = false;
            }
            Basis.setWindowPositionFromRegistry(this, Companyregistry.C_REG_ADR + "\\position", "table" + dbtable);
        }

        private void setGUI() {

            datagridDept.ToolTip = "Auswahl mit einem Doppelklick";
            datagridDept.IsReadOnly = true;  // Änderung
            datagridDept.CanUserAddRows = false;
            datagridDept.CanUserDeleteRows = false;
            datagridDept.CanUserResizeColumns = true;
            datagridDept.CanUserSortColumns = true;

            datagridEmpl.IsReadOnly = true;  // Änderung
            datagridEmpl.CanUserAddRows = false;
            datagridEmpl.CanUserDeleteRows = false;
            datagridEmpl.CanUserResizeColumns = true;
            datagridEmpl.CanUserSortColumns = true;

            CommandBindings.Add(new CommandBinding(MyCommands.NewDBFile, NewDBFileExecuted));
            CommandBindings.Add(new CommandBinding(MyCommands.OpenDBFile, OpenDBFileExecuted));

            CommandBindings.Add(new CommandBinding(MyCommands.Employee1, Employee1Executed, CheckCanExecuted));
            CommandBindings.Add(new CommandBinding(MyCommands.Employee2, Employee2Executed, CheckCanExecuted));
            CommandBindings.Add(new CommandBinding(MyCommands.Employee3, Employee3Executed, CheckCanExecuted));
            CommandBindings.Add(new CommandBinding(MyCommands.Department1, Department1Executed, CheckCanExecuted));
            CommandBindings.Add(new CommandBinding(MyCommands.Department2, Department2Executed, CheckCanExecuted));
            CommandBindings.Add(new CommandBinding(MyCommands.MasterDetails, MasterDetailsExecuted, CheckCanExecuted));

            CommandBindings.Add(new CommandBinding(MyCommands.Analyse, AnalyseExecuted, CheckCanExecuted));
            CommandBindings.Add(new CommandBinding(MyCommands.ExportXMLJSon, ExportXMLJSonExecuted, CheckCanExecuted));

            CommandBindings.Add(new CommandBinding(MyCommands.ImportAscii, ImportAsciiExecuted, CheckCanExecuted));
            CommandBindings.Add(new CommandBinding(MyCommands.ImportXML, ImportXMLExecuted, CheckCanExecuted));

            // neu: DatenbankKonsole
            CommandBindings.Add(new CommandBinding(MyCommands.DBS_Console, DBS_ConsoleExecuted, CheckCanExecuted));

            CommandBindings.Add(new CommandBinding(MyCommands.Openlastdatabasefile, OpenlastdatabasefileExecuted));
            CommandBindings.Add(new CommandBinding(MyCommands.Options, OptionsExecuted));

            CommandBindings.Add(new CommandBinding(MyCommands.TreeDepartment1, TreeDepartment1Executed, CheckCanExecuted));
            CommandBindings.Add(new CommandBinding(MyCommands.TreeDepartment2, TreeDepartment2Executed, CheckCanExecuted));
            CommandBindings.Add(new CommandBinding(MyCommands.TreeDepartment3, TreeDepartment3Executed, CheckCanExecuted));
            CommandBindings.Add(new CommandBinding(MyCommands.TreeDepartment4, TreeDepartment4Executed, CheckCanExecuted));
            setLastdatabase();
        }  // setGUI


        private void setLastdatabase() {
            registryfilename = Basis.ReadRegString(Companyregistry.C_REG_ADR, Companyregistry.C_REG_PRJNAME);
            if (!registryfilename.Equals("") && File.Exists(registryfilename)) {
                tLastdatabasefile.Text =  Basis.getFilename(registryfilename);
                tLastdatabasefile.ToolTip = registryfilename;
            }
            else {
                tLastdatabasefile.IsEnabled = false;
                bnOpenLastDatenbase.IsEnabled = false;
            }
        }

        private void CheckCanExecuted(Object sender, CanExecuteRoutedEventArgs e) {
            e.CanExecute = databasefile != null;
        }
        private void OpenlastdatabasefileExecuted(Object sender, ExecutedRoutedEventArgs e) {
            if (!registryfilename.Equals("")) {
                openDataBase(registryfilename);
            }
        }

        private void Employee1Executed(Object sender, ExecutedRoutedEventArgs e) {
            // Dialog Employee !
            WEmployee1 dialog = new WEmployee1(this);
            dialog.ShowDialog();
        }

        private void Employee2Executed(Object sender, ExecutedRoutedEventArgs e) {
            // Dialog Employee !
            WEmployee2 dialog = new WEmployee2(this);
            dialog.ShowDialog();
        }
        private void Employee3Executed(Object sender, ExecutedRoutedEventArgs e) {
            WEmployee3 dialog = new WEmployee3(this);
            dialog.ShowDialog();
        }

        // Neu
        private void Department1Executed(object sender, ExecutedRoutedEventArgs e) {
            (new WDepartment1(this)).ShowDialog();
        }
        private void Department2Executed(object sender, ExecutedRoutedEventArgs e) {
            (new WDepartment2(this)).ShowDialog();
        }

        private void MasterDetailsExecuted(object sender, ExecutedRoutedEventArgs e) {
            (new WMasterDetails(this)).ShowDialog();
        }


        private void AnalyseExecuted(object sender, ExecutedRoutedEventArgs e) {
            (new WAnalyse(this)).ShowDialog();
        }
        private void ExportXMLJSonExecuted(object sender, ExecutedRoutedEventArgs e) {
            (new WDialogExportXMLJSon(this)).ShowDialog();
        }

        private void ImportAsciiExecuted(object sender, ExecutedRoutedEventArgs e) {
            Basis.Message("not implemented", "yet");
            // (new WDialogExportXMLJSon(this)).ShowDialog();
        }
        private void ImportXMLExecuted(object sender, ExecutedRoutedEventArgs e) {
            (new WDialogImportXML(this)).ShowDialog();
        }

        

        private void TreeDepartment1Executed(object sender, ExecutedRoutedEventArgs e) {
            (new WTreeDepartment1(this)).ShowDialog();
        }
        private void TreeDepartment2Executed(object sender, ExecutedRoutedEventArgs e) {
            (new WTreeDepartment2(this)).ShowDialog();
        }
        private void TreeDepartment3Executed(object sender, ExecutedRoutedEventArgs e) {
            (new WTreeDepartment3(this)).ShowDialog();
        }
        private void TreeDepartment4Executed(object sender, ExecutedRoutedEventArgs e) {
            (new WTreeDepartment4(this)).ShowDialog();
        }


        private void NewDBFileExecuted(object sender, ExecutedRoutedEventArgs e) {
            String filename = "";
            SaveFileDialog savefiledialog = new SaveFileDialog();
            savefiledialog.Filter = "Datenbankdateien (*.fdb)|*.fdb|Alle Dateien|*.*";
            savefiledialog.DefaultExt = ".fdb";
            savefiledialog.FilterIndex = 1; // zählt von 1
            savefiledialog.InitialDirectory = Environment.CurrentDirectory;
            if (savefiledialog.ShowDialog() == true) {
                filename = savefiledialog.FileName;
                Dbs dbs = Dbs.getInstance();
                dbs.createDatabase(filename);
                dbs.setDbs(filename);
                WProgressBarCreateTables dialog = new WProgressBarCreateTables(this, Basis.getProgPath()+"db_company.txt");         
                dialog.ShowDialog();
            }
        }


        private void OpenDBFile_Click(Object sender, RoutedEventArgs e) {
            OpenDBFileExecuted(null, null);
        }

        private void OpenDBFileExecuted(Object sender, ExecutedRoutedEventArgs e) {
            OpenFileDialog openfiledialog = new OpenFileDialog();
            openfiledialog.Filter = "Datenbankdateien (*.fdb)|*.fdb|Alle Dateien|*.*";
            openfiledialog.DefaultExt = ".fdb";
            openfiledialog.FilterIndex = 1; // zählt von 1
            openfiledialog.InitialDirectory =
                Environment.CurrentDirectory;
            if (openfiledialog.ShowDialog() == true) {
               openDataBase(openfiledialog.FileName);
            }

        }


        private void openDataBase(String filename) {
            databasefile = filename;
            Companyregistry.setDatabasePath(Basis.getPath(filename));
            Basis.WriteRegString(Companyregistry.C_REG_ADR,
                Companyregistry.C_REG_PRJNAME, filename);
            tLastdatabasefile.Text = Basis.getFilename(filename);
            tLastdatabasefile.ToolTip = filename;
            tLastdatabasefile.Foreground = Brushes.Blue;

            this.Title = "Firma: " + Basis.getFilename(filename);
            Dbs dbs = Dbs.getInstance();
            dbs.Close();
            dbs.setDbs(filename);
            setDeptFromDatabase();
        }

        private void bnOpenLastKataster_Click(object sender, RoutedEventArgs e) {
            if (!registryfilename.Equals("")) {
                openDataBase(registryfilename);
            }
        }



        private void setDeptFromDatabase() {
            ObservableCollection_Dept dept_Class = Dbs_QueriesExecute.getItemsFromDept();
            dept_Items = dept_Class.Items;
            datagridDept.ItemsSource = dept_Items;
        }
        private void datagridDept_SelectionChanged(Object sender, System.Windows.Controls.SelectionChangedEventArgs e) {
            if (datagridDept.SelectedIndex >= 0) {
                ModelDept dept = dept_Items[datagridDept.SelectedIndex];

                datagridEmpl.ItemsSource = Dbs_QueriesExecute.getItemsFromEmployees(false, dept.Pindex).Items;
                datagridEmpl.Columns[0].Width = 100;
                datagridEmpl.Columns[1].Width = 200;
                datagridEmpl.Columns[2].Width = 200;

            }
        }

        private void DBS_ConsoleExecuted(object sender, ExecutedRoutedEventArgs e) {
            (new WDBConsole(this)).ShowDialog();
        }


        private void OptionsExecuted(Object sender, ExecutedRoutedEventArgs e) {
            (new WOptions(this)).ShowDialog();
        }

        // neu: ProgInfo  mit Windows-Interna !!!!
        private void bnProgInfo_Click(Object sender, RoutedEventArgs e) {
            (new WProgInfo(this)).ShowDialog();
        }

        private void Window_Closing(Object sender, System.ComponentModel.CancelEventArgs e) {
            Basis.writeWindowPosition2Registry(this, Companyregistry.C_REG_ADR + "\\position", "table" + dbtable);
        }

    }
}
