﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;


using System.Data;
using System.ComponentModel;
using Company.DBS_Klassen;
using Company.Models;

using Excel = Microsoft.Office.Interop.Excel;
using System.Diagnostics;
using System.Threading;
using Microsoft.Win32;
using System.IO;

namespace Company.Views {
    /// <summary>
    /// Interaction logic for WAnalyse.xaml
    /// </summary>
    public partial class WAnalyse : Window {

        private String dbtable = "WAnalyse";

        private Dbs dbs = Dbs.getInstance();
        private DataTable dtResult = null;
        private DataTable dtResultSQL = null;

        private String lastTableCaption = "";


        public WAnalyse(Window owner) {
            InitializeComponent();
            this.Owner = owner;
            this.WindowStartupLocation = WindowStartupLocation.CenterOwner;

        }

        private void Window_Loaded(Object sender, RoutedEventArgs e) {
            cbAnalyse.Items.Clear();
            cbAnalyse.Items.Add("1): Anzeige der Mitarbeiter mit den Filtern");
            cbAnalyse.Items.Add("2): Anzeige der Mitarbeiter, Längen gruppiert nach den Abteilungen");
            cbAnalyse.Items.Add("3): Summe der Gehälter, Längen gruppiert nach den Abteilungen");
            cbAnalyse.SelectedIndex = 0;

            datagrid.IsReadOnly = true;
            datagrid.CanUserAddRows = false;
            datagrid.CanUserDeleteRows = false;
            datagrid.CanUserResizeColumns = true;
            datagrid.CanUserSortColumns = true;

            datagridSQL.IsReadOnly = true;
            datagridSQL.CanUserAddRows = false;
            datagridSQL.CanUserDeleteRows = false;
            datagridSQL.CanUserResizeColumns = true;
            datagridSQL.CanUserSortColumns = true;

            // setEnabled(false);
            tLastname.Text = Basis.ReadRegString(Companyregistry.C_REG_ADR, "lastnamefilter");
            tMinSalary.Text = Basis.ReadRegString(Companyregistry.C_REG_ADR, "minsalary");
            tMaxSalary.Text = Basis.ReadRegString(Companyregistry.C_REG_ADR, "maxsalary");

            setFontsize();
            Basis.setWindowPositionFromRegistry(this, Companyregistry.C_REG_ADR + "\\position", "table" + dbtable);
        }

        // Neu: setzen der Schriftgrößen aus CompanyRegistry
        private void setFontsize() {
            int fHeadersize = Companyregistry.HeaderFont;
            int fNormalsize = Companyregistry.NormalFont;
            this.FontSize = fNormalsize;
            // labelTitle.FontSize = fHeadersize;
        }

        private void setEnabled(bool enabled) {
            cbAnalyse.IsEnabled = enabled;
            tLastname.IsEnabled = enabled;
            tMinSalary.IsEnabled = enabled;
            tMaxSalary.IsEnabled = enabled;
            datagrid.IsEnabled = enabled;
            bnRunSQL.IsEnabled = enabled;
        }

        private void cbAnalyse_SelectionChanged(Object sender, SelectionChangedEventArgs e) {            
            if (cbAnalyse.SelectedIndex >= 0) {
                Start_SQL();
            }
        }



        private void bnOpenFile_Click(Object sender, RoutedEventArgs e) {
            OpenFileDialog openFileDialog1 = new OpenFileDialog();
            openFileDialog1.Filter = "Textdateien (*.txt)|*.txt|Alle Dateien (*.*)|*.*";
            openFileDialog1.FilterIndex = 1;// zählt von 1 !
            openFileDialog1.DefaultExt = ".txt";
            openFileDialog1.InitialDirectory = Environment.CurrentDirectory;
            openFileDialog1.Multiselect = false;
            openFileDialog1.RestoreDirectory = true;
            if (openFileDialog1.ShowDialog() == true) {
                tSQLInput.Text = File.ReadAllText(openFileDialog1.FileName.ToString());
            }
        }

        private void bnSaveFile_Click(Object sender, RoutedEventArgs e) {
            SaveFileDialog saveFileDialog1 = new SaveFileDialog();
            saveFileDialog1.Filter = "Textdateien (*.txt)|*.txt|Alle Dateien (*.*)|*.*";
            saveFileDialog1.FilterIndex = 1;// zählt von 1 !
            saveFileDialog1.DefaultExt = ".txt";
            saveFileDialog1.InitialDirectory = Environment.CurrentDirectory;
            saveFileDialog1.RestoreDirectory = true;
            if (saveFileDialog1.ShowDialog() == true) {
                File.WriteAllText(saveFileDialog1.FileName.ToString(), tSQLInput.Text);
            }
        }

        private void bnStart_Click(Object sender, RoutedEventArgs e) {
            startSQL();
        }


        private void startSQL() {
            int isql = cbAnalyse.SelectedIndex + 1;
            if (isql < 0) {
                return;
            }
            Basis.WriteRegString(Companyregistry.C_REG_ADR, "lastnamefilter", tLastname.Text.Trim());
            Basis.WriteRegString(Companyregistry.C_REG_ADR, "minsalary", tMinSalary.Text.Trim());
            Basis.WriteRegString(Companyregistry.C_REG_ADR, "maxsalary", tMaxSalary.Text.Trim());


            String[] nachnamenfilter = null;
            String dummy = tLastname.Text.Trim();
            if (dummy.Length > 0) {
                nachnamenfilter = dummy.Split(new Char[] { ';' });
            }
            decimal minsalary = -1.0m;
            decimal maxsalary = -1.0m;
            decimal d;
            if (Check.tryDecimal(tMinSalary, out d)) {
                if (d > 0) {
                    minsalary = d;
                }
            }
            if (Check.tryDecimal(tMaxSalary, out d)) {
                if (d > 0) {
                    maxsalary = d;
                }
            }
            StringBuilder sbWhere = new StringBuilder();
            switch (isql) {
                case 1:
                    break;
                case 2:
                case 3:
                    sbWhere.AppendLine(" WHERE ("+ Dbs_QueriesExecute.TBL_Employee+".deptindex="+ Dbs_QueriesExecute.TBL_Department+".Pindex)");
                    break;
                default:
                    return;
            }  // switch


            if (minsalary >= 0.0m) {
                if (sbWhere.Length==0)
                    sbWhere.AppendLine(" WHERE (MONTHLYSALARY>=" + minsalary + ") ");
                else
                    sbWhere.AppendLine(" and (MONTHLYSALARY>=" + minsalary + ") ");
            }
            if (maxsalary >= 0.0m) {
                if (sbWhere.Length == 0)
                    sbWhere.AppendLine(" WHERE (MONTHLYSALARY<=" + maxsalary + ") ");
                else
                    sbWhere.AppendLine(" and (MONTHLYSALARY<=" + maxsalary + ") ");
            }
            if (nachnamenfilter!=null) {
                if (sbWhere.Length == 0)
                    sbWhere.AppendLine(" WHERE ( ");
                else
                    sbWhere.AppendLine(" and ( ");
                String item;
                for (int i = 0; i < nachnamenfilter.Length; i++) {
                    item = nachnamenfilter[i];
                    if (i > 0) {
                        sbWhere.Append(" OR ");
                    }
                    if (item.EndsWith("*")) {
                        sbWhere.AppendLine(" (Lastname LIKE '" + item.Substring(0, item.Length - 1) + "%' ) ");
                    }
                    else {
                        sbWhere.AppendLine(" (Lastname= '" + item + "' ) ");
                    }
                }
                sbWhere.AppendLine(" ) ");
            }
            int salarycolumn = -1;
            String sql = "";
            switch (isql) {
                
                case 1:
                    //cbAnalyse.Items.Add("1): Anzeige der Mitarbeiter mit den Filtern");
                    lastTableCaption = "Employee";
                    salarycolumn = 3;
                    sql = "SELECT Pindex, firstname, lastname, MONTHLYSALARY, street, ZIPCODE, city, country, mobile, rem, deptindex \r\n" +
                        " FROM "  + Dbs_QueriesExecute.TBL_Employee + " \r\n"+
                        sbWhere.ToString() + "\r\n" +
                        "order by lastname, firstname";
                    break;
                case 2:
                    //cbAnalyse.Items.Add("2): Anzeige der Mitarbeiter, Längen gruppiert nach den Abteilungen");
                    salarycolumn = 4;
                    lastTableCaption = "Employee";
                    sql = "SELECT "+ Dbs_QueriesExecute.TBL_Department+ ".city, \r\n" +
                               Dbs_QueriesExecute.TBL_Employee+ ".Pindex, firstname, lastname, MONTHLYSALARY, street, ZIPCODE, \r\n" + 
                               Dbs_QueriesExecute.TBL_Employee + ".city, "+Dbs_QueriesExecute.TBL_Employee + ".country, mobile, deptindex \r\n" +
                        " FROM " + Dbs_QueriesExecute.TBL_Employee + ", "+ Dbs_QueriesExecute.TBL_Department+ " \r\n" +
                        sbWhere.ToString() + "\r\n" +                        
                        " order by "+ Dbs_QueriesExecute.TBL_Department + ".city, lastname, firstname";
                    break;
                case 3:
                    //cbAnalyse.Items.Add("3): Summe der Gehälter, Längen gruppiert nach den Abteilungen");
                    salarycolumn = 1;
                    lastTableCaption = "Department";
                    sql = "SELECT " + Dbs_QueriesExecute.TBL_Department + ".city, Sum(MONTHLYSALARY) \r\n" +
                        " FROM " + Dbs_QueriesExecute.TBL_Employee + ", " + Dbs_QueriesExecute.TBL_Department + " " +
                        sbWhere.ToString() + "\r\n" +
                        " GROUP BY " + Dbs_QueriesExecute.TBL_Department + ".city " +
                        " order by "+ Dbs_QueriesExecute.TBL_Department + ".city ";
                    break;
                default:
                    return;
            }  // switch
            tSQLInput.Text = sql;
            dtResult = dbs.selectDBS(sql);
            if (dtResult==null) {
                // Fehler in sql
                datagrid.ItemsSource = null;
                labelCount.Content = "Anzahl der Daten: 0";
                return;
            }
            else {
                datagrid.ItemsSource = null;
                datagrid.ItemsSource = dtResult.DefaultView;
                labelCount.Content = "Anzahl der Daten: " + dtResult.Rows.Count.ToString();

                decimal summe = 0.0m;
                foreach (System.Data.DataRow row in dtResult.Rows) {
                    summe+= (Decimal)row.ItemArray[salarycolumn];
                }
                labelSummSalary.Content = summe.ToString("0.0");


            }



        }  // startSQL




        private void bnRunSQL_Click(Object sender, RoutedEventArgs e) {
            Start_SQL();
        }

        // Datenbank-Konsole
        private void Start_SQL() {
            string sql = tSQLInput.Text.Trim();
            if (sql.Equals("")) return;
            if (sql.ToLower().IndexOf("create") >= 0 || sql.ToLower().IndexOf("alter") >= 0 ||
                sql.ToLower().IndexOf("drop") >= 0 ||
                sql.ToLower().IndexOf("update") >= 0 || sql.ToLower().IndexOf("delete") >= 0) {
                Basis.Message("Die SQL-Anweisungen sind nicht gültig.\r\n(create, alter, drop, update, delete)", "Hinweis");
                return;
            }
            try {
                dtResultSQL = dbs.selectDBS(sql);
                if (dtResultSQL==null) {
                    datagridSQL.ItemsSource = null;
                    labelSummSalary.Content = "";
                    labelCount.Content = "Anzahl der Daten: 0";
                }
                else {
                    datagridSQL.ItemsSource = null;
                    datagridSQL.ItemsSource = dtResultSQL.DefaultView;
                    labelSummSalary.Content = "";
                    labelCount.Content = "Anzahl der Daten: " + dtResult.Rows.Count.ToString();
                }
            }
            catch (Exception e) {
                Basis.ErrorMsg("Fehlerhafte SQL-Anweisung\r\n" + e.ToString(), "Fehler");
            }
        }

        private void bnExcel_Click(Object sender, RoutedEventArgs e) {
            switch (tabcontrol.SelectedIndex) {
                case 0:
                    if (dtResult != null) {
                        Dt2Export.Export2Excel(dtResult);
                    }
                    break;
                case 1:
                    if (dtResultSQL != null) {
                        Dt2Export.Export2Excel(dtResultSQL);
                    }
                    break;
            }
        }

        private void bnJSon_Click(Object sender, RoutedEventArgs e) {

        }

        private void bnXML_Click(Object sender, RoutedEventArgs e) {
            if (dtResult==null) {
                Basis.Message("Bitte führen Sie eine gültige Abfrage aus", "Hinweis");
                return;
            }

            SaveFileDialog savefiledialog = new SaveFileDialog();
            savefiledialog.Title = "Export in eine XML-Datei";
            savefiledialog.Filter = "XML-Dateien (*.xml)|*.xml|Alle Dateien|*.*";
            savefiledialog.DefaultExt = ".xml";
            savefiledialog.FilterIndex = 1; // zählt von 1
            savefiledialog.InitialDirectory = Environment.CurrentDirectory;
            if (savefiledialog.ShowDialog() == true) {
                Dt2Export.Export2XML(savefiledialog.FileName, "Company", lastTableCaption,dtResult);

                //StringBuilder sb = new StringBuilder();
                //sb.AppendLine("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
                //sb.AppendLine("<Company xsi:noNamespaceSchemaLocation=\"company.xsd\"");
                //sb.AppendLine("xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\">");

                ////sb.AppendLine("\t<Employee>");
                ////sb.AppendLine("\t\t<Pindex>" + pindex + "</Pindex>");
                ////sb.AppendLine("\t</Employee>");


                
                //DataColumnCollection cols = dtResult.Columns;
                //foreach (System.Data.DataRow dbRow in dtResult.Rows) {
                //    sb.AppendLine("\t<" + lastTableCaption + ">");
                //    Object[] objs = dbRow.ItemArray;  // aktuelle Zeile
                //    for (int i = 0; i < objs.Length; i++) {
                //        sb.AppendLine("\t\t<"+ cols[i].Caption + ">" + objs[i] + "</"+ cols[i].Caption + "> "); 
                //    }
                //    sb.AppendLine("\t<" + lastTableCaption + ">");
                //}
                

                //sb.AppendLine("</Company>");

                //progressbar.Value = 0;
                //File.WriteAllText(savefiledialog.FileName, sb.ToString());

            }
        }  // save XML1
    

        private void Window_KeyDown(Object sender, KeyEventArgs e) {

        }

        private void bnEsc_Click(Object sender, RoutedEventArgs e) {
            Close();
        }

        private void Window_Closing(Object sender, CancelEventArgs e) {
            Basis.writeWindowPosition2Registry(this, Companyregistry.C_REG_ADR + "\\position", "table" + dbtable);
        }

    }
}
