﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;


using System.Collections.ObjectModel;
using System.Data;
using System.ComponentModel;
using System.Collections.Specialized; // ICollectionView
using Company.UserControls;
using Company.DBS_Klassen;
using Company.ViewModels;
using Company.Models;
using Company.ViewsDialog;

namespace Company.Views {
    /// <summary>
    /// Interaction logic for WDepartment2.xaml
    /// </summary>
    public partial class WDepartment2 : Window, INavigatorbar, IError  {

        private String dbtable = "WDepartment2";

        ObservableCollection_Dept dept_class;
        List<ModelDept> dept_Items;
        String filter = "";


        public WDepartment2(Window owner) {
            InitializeComponent();
            this.Owner = owner;
            this.WindowStartupLocation = WindowStartupLocation.CenterOwner;


            this.dept_class = Dbs_QueriesExecute.getItemsFromDept();
            this.dept_Items = dept_class.Items;
            datagrid.ItemsSource = dept_class.Items;  // one way

            datagrid.RowValidationRules.Add(new AbteilungValidationRule1(this, datagrid, dept_Items) {
                ValidationStep = ValidationStep.UpdatedValue
            });
        }





        private void Window_Loaded(object sender, RoutedEventArgs e) {
            navibar.setDatabaseParent(this, 1);
            int n = datagrid.Columns.Count;
            //navibar.setCount(dept_Items.Count);
            Title = "Anzahl: " + dept_Items.Count.ToString();

            datagrid.Columns[0].Width = 100;
            datagrid.Columns[2].Width = 50;
            datagrid.Columns[1].Width = 300; // datagrid.Width - 200;

            navibar.setCount(dept_class.Items.Count, dept_class.Items.Count);
            Basis.setWindowPositionFromRegistry(this, Companyregistry.C_REG_ADR + "\\position", "table" + dbtable);
        }



        // wird vom navibar aufgerufen
        public void setError(bool error) {
            bnOk.IsEnabled = !error;
        }
        

        public void setInsert(int nr) {
            ModelDept dept = new ModelDept();
            bool error = true;
            while (error) {
                error = false;
                WInputDept dialog = new WInputDept(this, true, dept);
                dialog.ShowDialog();
                if (!dialog.Retcode) return;
                // test auf doppelte Items
                String cityUpper = dept.City;
                String countryUpper = dept.Country;
                for (int i = 0; i < dept_Items.Count; i++) {
                    String sk = dept_Items[i].City.ToUpper();
                    if (dept_Items[i].City.ToUpper().Equals(cityUpper) && dept_Items[i].Country.ToUpper().Equals(countryUpper)) {
                        MessageBox.Show("Die Niederlassung ist schon vorhanden.", "Neuer Eintrag", MessageBoxButton.OK, MessageBoxImage.Error);
                        error = true;
                        break;
                    }
                }
                if (error) continue;

                error = false;
                dept_Items.Add(dept);
                setFilter(1, this.filter);
            }
        }  // setDelete

        public void setDelete(int nr) {
            ModelDept abt = (ModelDept)datagrid.SelectedItem;
            if (abt != null) {
                // check, ob die Abteilung noch benutzt wird?
                bool gefunden = Dbs_QueriesExecute.deptIsFKInEmplSale(abt.Pindex);
                if (gefunden) {
                    Basis.Message("Diese Abteilung darf nicht gelöscht werden, da sie noch verwendet wird", "Hinweis");
                }
                else {
                    if (Basis.readYesNo("Wollen Sie wirklich diese Abteilung löschen ?\r\n" +
                        "Abteilung: " + abt.City + " (" + abt.Country + ")", "Abteilung löschen")) {
                        dept_class.Delete(abt);
                        setFilter(1, this.filter);
                    }
                }
            }
        }



        String[] filteritems = null;
        public void setFilter(int nr, String filter) {
            this.filter = filter;
            string[] splitStrings = { ";" };
            filteritems = filter.Split(splitStrings, StringSplitOptions.None);
            int n = 0;
            foreach (ModelDept dept in dept_Items) {
                dept.isChecked = true;
                if (filter.Length > 0) {
                    foreach (String item in filteritems) {
                        if (!dept.City.StartsWith(item)) dept.isChecked = false;
                    }
                }
                if (dept.isChecked) {
                    n++;
                }
            }
            datagrid.ItemsSource = from dept in dept_Items
                                   where dept.isChecked
                                   select dept;
            navibar.setCount(n, dept_class.Items.Count);
        }

        private bool getModify() {
            bool modify = false;
            foreach (ModelDept abt in dept_Items) {
                if (abt.modifyAttrib) modify = true;
                if (abt.newAttrib) modify = true;
            }
            if (dept_class.getDeleteItemsCount() > 0) modify = true;
            return modify;
        }

        private void Window_Closing(object sender, CancelEventArgs e) {
            e.Cancel = false;
            if (getModify()) {
                MessageBoxResult res = Basis.readYesNoCancel("Sollen die Änderungen gespeichert werden?", "Dialog beenden");
                switch (res) {
                    case MessageBoxResult.Yes: dept_class.save2Database(); break;
                    case MessageBoxResult.No: break;
                    case MessageBoxResult.Cancel: e.Cancel = false; return;
                }
            }
            Basis.writeWindowPosition2Registry(this, Companyregistry.C_REG_ADR + "\\position", "table" + dbtable);

        }
        private void bnOk_Click(object sender, RoutedEventArgs e) {
            if (getModify()) {
                dept_class.save2Database();
            }
            Close();
        }


        private void bnEsc_Click(object sender, RoutedEventArgs e) {
            Close();
        }


        // neuerCode: Eingaben werden direkt entgegengenommen
        public class AbteilungValidationRule1 : ValidationRule {

            private DataGrid grid;
            private List<ModelDept> abtitems;
            private IError error;

            public AbteilungValidationRule1(IError error, DataGrid grid, List<ModelDept> abtitems) {
                this.error = error;
                this.grid = grid;
                this.abtitems = abtitems;
            }

            public override ValidationResult Validate(object value, System.Globalization.CultureInfo cultureInfo) {
                ModelDept abt = (value as BindingGroup).Items[0] as ModelDept;
                int index = grid.Items.IndexOf(abt);

                String abtUpper = abt.City.Trim().ToUpper();
                if (abtUpper.Length == 0) {
                    MessageBox.Show("Bitte geben Sie einen Kurzbezeichung ein. Eine leere Kurzbezeichung ist verboten.", "Eintrag Bearbeitung", MessageBoxButton.OK, MessageBoxImage.Error);
                    error.setError(true);
                    return new ValidationResult(false, "Bitte geben Sie eine Kurzbezeichung ein. Eine leerer Kurzbezeichung ist verboten.");
                }
                for (int i = 0; i < abtitems.Count; i++) {
                    String sk = abtitems[i].City.ToUpper();
                    if (i != index && abtitems[i].City.ToUpper().Equals(abtUpper)) {
                        MessageBox.Show("Die Kurzbezeichung ist schon vorhanden.", "Eintrag Bearbeitung", MessageBoxButton.OK, MessageBoxImage.Error);
                        error.setError(true);
                        return new ValidationResult(false, "Die Kurzbezeichung ist schon vorhanden.");
                    }
                }
                error.setError(false);
                return ValidationResult.ValidResult;
            }
        } // AbteilungValidationRule1



        private void datagrid_BeginningEdit(object sender, DataGridBeginningEditEventArgs e) {
            DataGridColumn col = e.Column;
            if (col.Header.Equals("PIndex")) {  // Pindex
                e.Cancel = true;
            }
        }

        private void datagrid_CellEditEnding(object sender, DataGridCellEditEndingEventArgs e) {
            DataGridColumn col = e.Column;
            e.Cancel = false;
            if (col.Header.Equals("KurzBez")) {
                TextBox t = e.EditingElement as TextBox;
                DataGridEditAction action = e.EditAction;
                String item = t.Text.Trim();
                if (item.Length == 0) e.Cancel = true;

            }
            if (col.Header.Equals("Bez")) {
                TextBox t = e.EditingElement as TextBox;
                DataGridEditAction action = e.EditAction;
                String item = t.Text.Trim();
                if (item.Length == 0) e.Cancel = true;
            }

        }




    }
}
