﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;


using System.Collections.ObjectModel;
using System.Data;
using System.ComponentModel;
using System.Collections.Specialized; // ICollectionView
using Company.UserControls;
using Company.DBS_Klassen;
using Company.ViewModels;
using Company.Models;
using Company.ViewsDialog;

using Excel = Microsoft.Office.Interop.Excel;
using System.Diagnostics;


namespace Company.Views {
    /// <summary>
    /// Interaction logic for WEmployee3.xaml
    /// </summary>
    public partial class WEmployee3 : Window, INavigatorbar {

        private String dbtable = "WEmployee3";

        ObservableCollection_Employee empl_class;
        List<ModelEmployee> empl_Items;

        ICollectionView collView = null;
        String filter = "";
        
        public WEmployee3(Window owner) {
            InitializeComponent();
            this.Owner = owner;
            this.WindowStartupLocation = WindowStartupLocation.CenterOwner;

            empl_class = Dbs_QueriesExecute.getItemsFromEmployees(false);
            this.empl_Items = empl_class.Items;

        }


        private void Window_Loaded(object sender, RoutedEventArgs e) {
            navibar.setDatabaseParent(this, 1);

            datagrid.ItemsSource = empl_class.Items;  // one way
            collView = CollectionViewSource.GetDefaultView(datagrid.ItemsSource);
            collView.SortDescriptions.Add(new SortDescription("Lastname", ListSortDirection.Ascending));
            collView.SortDescriptions.Add(new SortDescription("Firstname", ListSortDirection.Ascending));

            int n = datagrid.Columns.Count;
            //navibar.setCount(empl_Items.Count);
            Title = "Employee-Dialog";
            LMitarbeiter.Content = "Employee: ";

            datagrid.Columns[0].Width = 100;
            datagrid.Columns[1].Width = 200; // datagrid.Width - 200;
            datagrid.Columns[2].Width = 50;

            datagrid.CanUserAddRows = false;
            datagrid.CanUserDeleteRows = false;
            datagrid.CanUserResizeColumns = true;
            datagrid.CanUserSortColumns = true;
            datagrid.CanUserAddRows = false;
            navibar.setCount(empl_class.Items.Count, empl_class.Items.Count);
            Basis.setWindowPositionFromRegistry(this, Companyregistry.C_REG_ADR + "\\position", "table" + dbtable);
        }


        
        public void setInsert(int nr) {
            ModelEmployee empl = new ModelEmployee();
            WInputEmployee dialog = new WInputEmployee(this, true, empl);
            dialog.ShowDialog();
            if (dialog.Retcode) {
                empl_Items.Add(empl);
                setFilter(1, this.filter);
            }
        }

        public void setDelete(int nr) {
            ModelEmployee empl = (ModelEmployee)datagrid.SelectedItem;
            if (empl != null) {
                if (Basis.readYesNo("Wollen Sie wirklich diesen Mitarbeiter löschen ?\r\n" +
                        "Mitarbeiter: " + empl.Lastname + ", " + empl.Firstname + ")", "Mitarbeiter löschen")) {
                    empl_class.Delete(empl);
                    setFilter(1, this.filter);
                }
            }
        }


        String[] filteritems = null;
        public void setFilter(int nr, String filter) {
            this.filter = filter;
            string[] splitStrings = { ";" };
            filteritems = filter.Split(splitStrings, StringSplitOptions.None);
            int n = 0;
            foreach (ModelEmployee empl in empl_Items) {
                empl.isChecked = true;
                if (filter.Length > 0) {
                    foreach (String item in filteritems) {
                        if (!empl.Lastname.StartsWith(item)) empl.isChecked = false;
                    }
                }
                if (empl.isChecked) {
                    n++;
                }
            }
            datagrid.ItemsSource = from empl in empl_Items
                                   where empl.isChecked
                                   select empl;
            navibar.setCount(n, empl_Items.Count);
        }



        // Spalten, die nicht editierbar sein sollen
        private void datagrid_BeginningEdit(object sender, DataGridBeginningEditEventArgs e) {
            DataGridColumn col = e.Column;
            if (col.Header.Equals("PIndex") || col.Header.Equals("AIndex") || col.Header.Equals("Jahresgehalt")) {
                e.Cancel = true;
            }
        }



        private void datagrid_CellEditEnding(object sender, DataGridCellEditEndingEventArgs e) {
            DataGridColumn col = e.Column;
            e.Cancel = false;
            if (col.Header.Equals("Lastname")) {
                TextBox t = e.EditingElement as TextBox;
                //DataGridEditAction action = e.EditAction;
                String item = t.Text.Trim();
                if (item.Length == 0) e.Cancel = true;

            }
            if (col.Header.Equals("Firstname")) {
                TextBox t = e.EditingElement as TextBox;
                //DataGridEditAction action = e.EditAction;
                String item = t.Text.Trim();
                if (item.Length == 0) e.Cancel = true;
            }
            if (col.Header.Equals("Monthsalary")) {
                TextBox t = e.EditingElement as TextBox;
                decimal monatsgehalt;
                if (decimal.TryParse(t.Text.Trim(), System.Globalization.NumberStyles.AllowDecimalPoint,
                           System.Globalization.CultureInfo.InvariantCulture, out monatsgehalt)) {
                    DataGridRow row = e.Row;

                    ModelEmployee empl = (ModelEmployee)row.Item;
                    empl.Yearsalary = monatsgehalt * 12.0m;
                    System.Diagnostics.Debug.WriteLine("datagrid_CellEditEnding: Jahresgehalt: " + empl.Yearsalary);
                    empl.updateYearsalary();
                }
            }
            System.Diagnostics.Debug.WriteLine("datagrid_CellEditEnding");
        }



        private void datagrid_SelectionChanged(object sender, SelectionChangedEventArgs e) {
            if (datagrid.SelectedIndex >= 0) {
                ModelEmployee mit = empl_Items[datagrid.SelectedIndex];
                LMitarbeiter.Content = mit.Lastname + ", " + mit.Firstname;
            }
        }

        private void bnTest_Click(object sender, RoutedEventArgs e) {
            ModelEmployee mit = empl_Items[2];
            mit.Monthlysalary = 2000;
            LMitarbeiter.Content = "Test: " + mit.Lastname + ", " + mit.Firstname;
            collView.Refresh();
        }



        private void bnExport2Excel1_Click(object sender, RoutedEventArgs e) {
            Stopwatch stopWatch1 = new Stopwatch();
            stopWatch1.Start();
            var xl = new Excel.Application();
            var wb = xl.Workbooks.Add();

            int anzCols = ModelEmployee.CaptionItemArray.Length;
            for (int i = 0; i < anzCols; i++) {
                xl.Cells[1, i + 1] = ModelEmployee.CaptionItemArray[i]; //Row,Column
                xl.Cells[1, i + 1].Select();
                xl.ActiveCell.Font.Size = 12;
                xl.ActiveCell.Font.Bold = true;
            }

            var ws = (Microsoft.Office.Interop.Excel.Worksheet)wb.ActiveSheet;

            int n = 0;
            int row = 0;
            ws.Range["A2:B" + (empl_Items.Count + 1)].Select();
            xl.Selection.NumberFormat = "@";  // String

            foreach (ModelEmployee emp in empl_Items) {
                if (emp.isChecked) {
                    row++;
                    Title = row + "  /  " + empl_Items.Count;
                    n++;
                    xl.Cells[row, 1] = (row - 1).ToString();  //Row,Column
                    xl.Cells[row, 2] = emp.Firstname;
                    xl.Cells[row, 3] = emp.Lastname;
                    xl.Cells[row, 4] = emp.Monthlysalary;
                    xl.Cells[row, 5] = emp.Yearsalary;
                    xl.Cells[row, 6] = emp.Street;
                    xl.Cells[row, 7] = emp.Zipcode;
                    xl.Cells[row, 8] = emp.City;
                    xl.Cells[row, 9] = emp.Country;
                }
            }
            stopWatch1.Stop();
            TimeSpan ts = stopWatch1.Elapsed;
            Basis.Message(String.Format("{0:00}:{1:00}:{2:00}.{3:00}", ts.Hours, ts.Minutes, ts.Seconds, ts.Milliseconds / 10), "Länge der Zeit");
            xl.Visible = true;

        }  // bnExport2Excel1_Click  


        private void bnExport2Excel2_Click(object sender, RoutedEventArgs e) {
            Stopwatch stopWatch1 = new Stopwatch();
            stopWatch1.Start();
            var xl = new Excel.Application();
            var wb = xl.Workbooks.Add();

            int anzCols = ModelEmployee.CaptionItemArray.Length;
            for (int i = 0; i < anzCols; i++) {
                xl.Cells[1, i + 1] = ModelEmployee.CaptionItemArray[i]; //Row,Column
                xl.Cells[1, i + 1].Select();
                xl.ActiveCell.Font.Size = 12;
                xl.ActiveCell.Font.Bold = true;
            }

            var ws = (Microsoft.Office.Interop.Excel.Worksheet)wb.ActiveSheet;

            int n = 0;
            int rowOffset = 0;
            int row = 0;
            int iIndex = 0;
            int MAX = 10;
            ws.Range["A2:B" + (empl_Items.Count + 1)].Select();
            xl.Selection.NumberFormat = "@";  // String

            Object[,] items = new Object[MAX, anzCols];
            foreach (ModelEmployee emp in empl_Items) {
                if (emp.isChecked) {
                    iIndex++;
                    Title = iIndex + "  /  " + empl_Items.Count;
                    n++;
                    emp.insertinItemArray(rowOffset, items);
                    rowOffset++;
                    if (n == MAX) {
                        ws.Range["A2"].Offset[row].Resize[MAX, anzCols].Value = items;
                        n = 0;
                        rowOffset = 0;
                        row += MAX;
                    }
                }
            }
            if (n > 0) {
                ws.Range["A2"].Offset[row].Resize[rowOffset, anzCols].Value = items;
            }
            stopWatch1.Stop();
            TimeSpan ts = stopWatch1.Elapsed;
            Basis.Message(String.Format("{0:00}:{1:00}:{2:00}.{3:00}", ts.Hours, ts.Minutes, ts.Seconds, ts.Milliseconds / 10), "Länge der Zeit");
            xl.Visible = true;
        }  // bnExport2Excel2_Click    



        private bool getModify() {
            bool modify = false;
            foreach (ModelEmployee mit in empl_Items) {
                if (mit.modifyAttrib) modify = true;
                if (mit.newAttrib) modify = true;
            }
            if (empl_class.getDeleteItemsCount() > 0) modify = true;
            return modify;
        }

        private void Window_Closing(object sender, CancelEventArgs e) {
            e.Cancel = false;
            if (getModify()) {
                MessageBoxResult res = Basis.readYesNoCancel("Sollen die Änderungen gespeichert werden?", "Dialog beenden");
                switch (res) {
                    case MessageBoxResult.Yes: empl_class.save2Database(); break;
                    case MessageBoxResult.No: break;
                    case MessageBoxResult.Cancel: e.Cancel = false; return;
                }
            }
            Basis.writeWindowPosition2Registry(this, Companyregistry.C_REG_ADR + "\\position", "table" + dbtable);

        }
        private void bnOk_Click(object sender, RoutedEventArgs e) {
            if (getModify()) {
                empl_class.save2Database();
            }
            Close();
        }


        private void bnEsc_Click(object sender, RoutedEventArgs e) {
            Close();
        }



    }
}
