﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;

using System.Collections.ObjectModel;
using System.Data;
using System.ComponentModel;
using System.Collections.Specialized; // ICollectionView
using Company.UserControls;
using Company.DBS_Klassen;
using Company.ViewModels;
using Company.Models;
using Company.ViewsDialog;

using Excel = Microsoft.Office.Interop.Excel;
using System.Diagnostics;
using System.Threading;
using Microsoft.Win32;


namespace Company.Views {
    /// <summary>
    /// Interaction logic for WMasterDetails.xaml
    /// </summary>
    public partial class WMasterDetails : Window, INavigatorbar, IError, INotifyPropertyChanged {

        ObservableCollection_Dept dept_class;
        List<ModelDept> dept_Items;
        String filter = "";

        ObservableCollection_Employee empl_class = null;
        List<ModelEmployee> empl_Items = null;


        private void NotifyPropertyChanged(string p) {
            if (PropertyChanged != null) {
                PropertyChanged(datagridDept, new PropertyChangedEventArgs(p));
            }
        }
        public event PropertyChangedEventHandler PropertyChanged;


        public WMasterDetails(Window owner) {
            InitializeComponent();
            this.Owner = owner;
            this.WindowStartupLocation = WindowStartupLocation.CenterOwner;


            this.dept_class = Dbs_QueriesExecute.getItemsFromDept();
            this.dept_Items = dept_class.Items;
            datagridDept.ItemsSource = dept_class.Items;  // one way

        }

        private void Window_Loaded(object sender, RoutedEventArgs e) {
            navibar.setDatabaseParent(this, 1);
            int n = datagridDept.Columns.Count;
            //navibar.setCount(dept_Items.Count);
            Title = "Anzahl: " + dept_Items.Count.ToString();

            datagridDept.Columns[0].Width = 100;
            datagridDept.Columns[2].Width = 50;
            datagridDept.Columns[1].Width = 300; // datagridDept.Width - 200;

            datagridDept.ToolTip = "Bearbeiten mit einem Doppelklick";
            datagridDept.IsReadOnly = true;  // Änderung
            datagridDept.CanUserAddRows = false;
            datagridDept.CanUserDeleteRows = false;
            datagridDept.CanUserResizeColumns = true;
            datagridDept.CanUserSortColumns = true;

            datagridEmpl.Columns[0].Width = 100;
            datagridEmpl.Columns[1].Width = 200;
            datagridEmpl.Columns[2].Width = 50;
            datagridEmpl.ToolTip = "Bearbeiten mit einem Doppelklick";
            datagridEmpl.IsReadOnly = true;  // Änderung
            datagridEmpl.CanUserAddRows = false;
            datagridEmpl.CanUserDeleteRows = false;
            datagridEmpl.CanUserResizeColumns = true;
            datagridEmpl.CanUserSortColumns = true;
        }



        // wird vom navibar aufgerufen
        public void setError(bool error) {
            bnOk.IsEnabled = !error;
        }
        

        private void datagridDept_SelectionChanged(object sender, SelectionChangedEventArgs e) {
            if (datagridDept.SelectedIndex >= 0) {
                ModelDept dept = dept_Items[datagridDept.SelectedIndex];
                LDepartment.Content = dept.City + ", " + dept.Country;
                empl_class = Dbs_QueriesExecute.getItemsFromEmployees(false, dept.Pindex);  // bool sortDept
                this.empl_Items = empl_class.Items;
                datagridEmpl.ItemsSource = empl_class.Items;  // one way
            }
        }


        public void setInsert(int nr) {
            ModelDept dept = new ModelDept();
            bool error = true;
            while (error) {
                error = false;
                WInputDept dialog = new WInputDept(this, true, dept);
                dialog.ShowDialog();
                if (!dialog.Retcode) return;
                // test auf doppelte Items
                String cityUpper = dept.City;
                String countryUpper = dept.Country;
                for (int i = 0; i < dept_Items.Count; i++) {
                    String sk = dept_Items[i].City.ToUpper();
                    if (dept_Items[i].City.ToUpper().Equals(cityUpper) && dept_Items[i].Country.ToUpper().Equals(countryUpper)) {
                        MessageBox.Show("Die Niederlassung ist schon vorhanden.", "Neuer Eintrag", MessageBoxButton.OK, MessageBoxImage.Error);
                        error = true;
                        break;
                    }
                }
                if (error) continue;

                error = false;
                dept_Items.Add(dept);
            }
        }

        private void datagridEmpl_MouseDoubleClick(object sender, MouseButtonEventArgs e) {
            int index = datagridEmpl.SelectedIndex;
            if (index >= 0) {
                bool error = true;
                ModelEmployee empl = (ModelEmployee)datagridEmpl.SelectedItem;
                int pindex = empl.Pindex;
                ModelEmployee empl2 = new ModelEmployee();
                while (error) {
                    empl2.init(empl, false);
                    WInputEmployee dialog = new WInputEmployee(this, false, empl2);
                    dialog.ShowDialog();
                    if (!dialog.Retcode) return;

                    error = false;
                    for (int i = 0; i < empl_Items.Count; i++) {
                        if (empl_Items[i].Pindex != empl2.Pindex &&
                               empl2.Lastname != empl_Items[i].Lastname && empl2.Firstname == empl_Items[i].Firstname) {
                            MessageBox.Show("Der Mitarbeiter ist schon vorhanden.", "Neuer Eintrag", MessageBoxButton.OK, MessageBoxImage.Error);
                            error = true;
                            break;
                        }
                    }
                    if (error) continue;
                    empl.init(empl2, true);
                    datagridEmpl.Items.Refresh();
                    datagridEmpl.Focus();
                }
            }
        }

        private void datagridDept_MouseDoubleClick(object sender, MouseButtonEventArgs e) {
            int index = datagridDept.SelectedIndex;
            if (index >= 0) {
                bool error = true;
                ModelDept dept = (ModelDept)datagridDept.SelectedItem;
                int pindex = dept.Pindex;
                ModelDept dept2 = new ModelDept();
                while (error) {
                    dept2.init(dept, false);
                    WInputDept dialog = new WInputDept(this, false, dept2);
                    dialog.ShowDialog();
                    if (!dialog.Retcode) return;

                    error = false;
                    for (int i = 0; i < dept_Items.Count; i++) {
                        if (dept_Items[i].Pindex != dept2.Pindex && dept2.City == dept_Items[i].City && dept2.Country == dept_Items[i].Country) {
                            MessageBox.Show("Die Niederlassung ist schon vorhanden.", "Eintrag bearbeiten", MessageBoxButton.OK, MessageBoxImage.Error);
                            error = true;
                            break;
                        }
                    }
                    if (error) continue;
                    dept.init(dept2, true);
                    datagridDept.Items.Refresh();
                    datagridDept.Focus();
                    datagridDept_SelectionChanged(null, null);
                }
            }
        }


        public void setDelete(int nr) {
            ModelDept abt = (ModelDept)datagridDept.SelectedItem;
            if (abt != null) {
                // check, ob die Abteilung noch benutzt wird?
                bool gefunden = Dbs_QueriesExecute.deptIsFKInEmplSale(abt.Pindex);
                if (gefunden) {
                    Basis.Message("Diese Abteilung darf nicht gelöscht werden, da sie noch verwendet wird", "Hinweis");
                }
                else {
                    if (Basis.readYesNo("Wollen Sie wirklich diese Abteilung löschen ?\r\n" +
                        "Abteilung: " + abt.City + " (" + abt.Country + ")", "Abteilung löschen")) {
                        dept_class.Delete(abt);
                        //navibar.setCount(dept_Items.Count);
                    }
                }
            }
        }

        public void setCancel(int nr) {
            ((DataRowView)datagridDept.SelectedItem).Row.CancelEdit();
        }

        String[] filteritems = null;
        public void setFilter(int nr, String filter) {
            this.filter = filter;
            string[] splitStrings = { ";" };
            filteritems = filter.Split(splitStrings, StringSplitOptions.None);
            foreach (ModelDept dept in dept_Items) {
                dept.isChecked = true;
                if (filter.Length > 0) {
                    foreach (String item in filteritems) {
                        if (!dept.City.StartsWith(item)) dept.isChecked = false;
                    }
                }
            }
            datagridDept.ItemsSource = from dept in dept_Items
                                   where dept.isChecked
                                   select dept;
        }


        private bool getModify() {
            bool modify = false;
            foreach (ModelDept abt in dept_Items) {
                if (abt.modifyAttrib) modify = true;
                if (abt.newAttrib) modify = true;
            }
            if (dept_class.getDeleteItemsCount() > 0) modify = true;
            return modify;
        }


        private void Window_Closing(object sender, CancelEventArgs e) {
            e.Cancel = false;
            if (getModify()) {
                MessageBoxResult res = Basis.readYesNoCancel("Sollen die Änderungen gespeichert werden?", "Dialog beenden");
                switch (res) {
                    case MessageBoxResult.Yes: dept_class.save2Database(); break;
                    case MessageBoxResult.No: break;
                    case MessageBoxResult.Cancel: e.Cancel = false; return;
                }
            }

        }
        private void bnOk_Click(object sender, RoutedEventArgs e) {
            if (getModify()) {
                dept_class.save2Database();
            }
            Close();
        }


        private void bnEsc_Click(object sender, RoutedEventArgs e) {
            Close();
        }



    }
}
