﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;


using System.ComponentModel;
using Company.DBS_Klassen;
using Company.ViewModels;
using Company.Models;
using System.Diagnostics;

namespace Company.Views {
    /// <summary>
    /// Interaction logic for WTreeDepartment1.xaml
    /// </summary>
    public partial class WTreeDepartment1 : Window {

        ObservableCollection_Dept dept_class;
        List<ModelDept> dept_Items;

        ObservableCollection_Employee empl_class;
        List<ModelEmployee> empl_Items;



        public WTreeDepartment1(Window owner) {
            InitializeComponent();
            this.Owner = owner;
            this.WindowStartupLocation = WindowStartupLocation.CenterOwner;

            this.dept_class = Dbs_QueriesExecute.getItemsFromDept();
            this.dept_Items = dept_class.Items;

            empl_class = Dbs_QueriesExecute.getItemsFromEmployees(false);
            this.empl_Items = empl_class.Items;
        }

        // es wird ein Dept-Knoten eingetragen und dann werden die dazugehörigen Employees eingefügt
        private void Window_Loaded(object sender, RoutedEventArgs e) {
            Title = "Anzahl: " + dept_Items.Count.ToString();
            tree.Items.Clear();
            TreeViewItem root;
            tree.ItemsSource = null;

            Stopwatch stopWatch = new Stopwatch();
            stopWatch.Start();

            root = new TreeViewItem();
            root.Header = "root";
            root.Foreground = Brushes.Red;
            root.IsExpanded = true;
            tree.Items.Add(root);
            TreeViewItem deptNode;
            foreach (ModelDept dept in dept_Items) {
                deptNode = insertDept2Tree(root, dept);
                foreach (ModelEmployee empl in empl_Items) {
                    if (empl.Deptindex == dept.Pindex) {
                        insertEmpl2Tree(deptNode, empl);
                    }
                }
            }

            stopWatch.Stop();
            TimeSpan ts = stopWatch.Elapsed;
            Basis.Message("Zeit: " + ts.ToString(), "Tree1");
        }
        

        private TreeViewItem insertDept2Tree(TreeViewItem parent, ModelDept dept) {
            TreeViewItem node = new TreeViewItem();
            node.Header = dept.City + ", " + dept.Country;
            node.Tag = dept;
            parent.Items.Add(node);
            return node;
        }

        private TreeViewItem insertEmpl2Tree(TreeViewItem parent, ModelEmployee empl) {
            TreeViewItem node = new TreeViewItem();
            node.Header = empl.ToString();
            node.Tag = empl;
            parent.Items.Add(node);
            return node;
        }


        private bool getModify() {
            bool modify = false;
            foreach (ModelDept abt in dept_Items) {
                if (abt.modifyAttrib) modify = true;
                if (abt.newAttrib) modify = true;
            }
            if (dept_class.getDeleteItemsCount() > 0) modify = true;
            return modify;
        }

        private void Window_Closing(object sender, CancelEventArgs e) {
            e.Cancel = false;
            if (getModify()) {
                MessageBoxResult res = Basis.readYesNoCancel("Sollen die Änderungen gespeichert werden?", "Dialog beenden");
                switch (res) {
                    case MessageBoxResult.Yes: dept_class.save2Database(); break;
                    case MessageBoxResult.No: break;
                    case MessageBoxResult.Cancel: e.Cancel = false; return;
                }
            }

        }
        private void bnOk_Click(object sender, RoutedEventArgs e) {
            if (getModify()) {
                dept_class.save2Database();
            }
            Close();
        }


        private void bnEsc_Click(object sender, RoutedEventArgs e) {
            Close();
        }




    }
}

