﻿
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;


using Microsoft.Win32;
using System.IO;
using Company.DBS_Klassen;
using System.Data;
using System.Diagnostics;
using System.Windows.Controls.Ribbon;

// using Excel = Microsoft.Office.Interop.Excel;

namespace Company.ViewsDialog {
    /// <summary>
    /// Interaction logic for FDBConsole.xaml
    /// </summary>
    public partial class WDBConsole : RibbonWindow {

        private DataTable dtDataBase;
        private bool start = true;
        private Dbs dbs = Dbs.getDbsInstance();
        DataTable dtResult = null;

        public WDBConsole(Window owner) {
            InitializeComponent();
            this.Owner = owner;
            this.WindowStartupLocation = WindowStartupLocation.CenterOwner;
        }

        private void Window_Loaded(object sender, RoutedEventArgs e) {
            dtDataBase = dbs.getTables();
            setTables("");
            datagrid.IsReadOnly = true;
            datagrid.CanUserAddRows = false;
            datagrid.CanUserDeleteRows = false;
            datagrid.CanUserResizeColumns = true;
            datagrid.CanUserSortColumns = true;
        }



        private void setTables(String filter) {
            start = true;
            listBox1.Items.Clear();
            for (int row = 0; row < dtDataBase.Rows.Count; row++) {
                string sTabletyp = (string)dtDataBase.Rows[row][3];
                if (sTabletyp.Equals("TABLE")) {
                    if (filter.Length > 0) {
                        if (dtDataBase.Rows[row][2].ToString().ToLower().StartsWith(filter)) {
                            listBox1.Items.Add(dtDataBase.Rows[row][2].ToString());
                        }
                    }
                    else {
                        listBox1.Items.Add(dtDataBase.Rows[row][2].ToString());
                    }
                }
            }
            if (listBox1.Items.Count > 0) listBox1.SelectedIndex = 0;
            start = false;
        }

        private void listBox1_SelectionChanged(object sender, SelectionChangedEventArgs e) {
            int i = listBox1.SelectedIndex;
            if ((!start) && (i >= 0)) {
                string sTable = (string)listBox1.SelectedItem;
                DataTable dtTablenames = dbs.getDataTyps(sTable);  // getDataTyps(string sTable)
                listBox2.Items.Clear();
                for (int row = 0; row < dtTablenames.Rows.Count; row++) {
                    string sAttribname = (string)dtTablenames.Rows[row][3];
                    string sFormat = (string)dtTablenames.Rows[row][4];
                    listBox2.Items.Add(sAttribname + " (" + sFormat + ")");
                }
            }
        }


        private void tFilterListeTabelle_TextChanged(object sender, TextChangedEventArgs e) {
            setTables(tFilterListeTabelle.Text.Trim().ToLower());
        }


        private void listBox1_MouseDoubleClick(object sender, MouseButtonEventArgs e) {
            int i = listBox1.SelectedIndex;
            if ((!start) && (i >= 0)) {
                string column = (string)listBox1.SelectedItem;
                if (Keyboard.Modifiers.HasFlag(ModifierKeys.Shift)) {
                    tInput.Text = "SELECT * FROM " + column.Trim();
                   
                }
                else {
                    Clipboard.SetText(" " + column.Trim());
                    tInput.Paste();
                    tInput.Focus();
                }
            }
        } // listBox1_MouseDoubleClick

        private void listBox2_MouseDoubleClick(object sender, MouseButtonEventArgs e) {
            int i = listBox2.SelectedIndex;
            if ((!start) && (i >= 0)) {
                string column = (string)listBox2.SelectedItem;
                int pos = column.IndexOf('(');
                if (i == -1) {
                    Clipboard.SetText(" " + column.Trim());
                }
                else {
                    if (i >= 0)
                        Clipboard.SetText(" " + column.Substring(0, pos - 1).Trim());
                    else
                        Clipboard.SetText(" " + column.Trim());
                }
                tInput.Paste();
                tInput.Focus();
            }
        } // listBox2_MouseDoubleClick


        private void OpenSQLText() {
            OpenFileDialog openFileDialog1 = new OpenFileDialog();
            openFileDialog1.Filter = "Textdateien (*.txt)|*.txt|Alle Dateien (*.*)|*.*";
            openFileDialog1.FilterIndex = 1;// zählt von 1 !
            openFileDialog1.DefaultExt = ".txt";
            openFileDialog1.InitialDirectory = Environment.CurrentDirectory;
            openFileDialog1.Multiselect = false;
            openFileDialog1.RestoreDirectory = true;
            if (openFileDialog1.ShowDialog() == true) {
                tInput.Text = File.ReadAllText(openFileDialog1.FileName.ToString());
            }
        }

        private void SaveSQLText() {
            SaveFileDialog saveFileDialog1 = new SaveFileDialog();
            saveFileDialog1.Filter = "Textdateien (*.txt)|*.txt|Alle Dateien (*.*)|*.*";
            saveFileDialog1.FilterIndex = 1;// zählt von 1 !
            saveFileDialog1.DefaultExt = ".txt";
            saveFileDialog1.InitialDirectory = Environment.CurrentDirectory;
            saveFileDialog1.RestoreDirectory = true;
            if (saveFileDialog1.ShowDialog() == true) {
                File.WriteAllText(saveFileDialog1.FileName.ToString(), tInput.Text);
            }
        }

        private void mnClose_Click(object sender, RoutedEventArgs e) {
            Close();
        }


        private void Start() {
            DateTime Time1, Time2;
            string sSQL = tInput.Text.Trim();
            if (sSQL.Equals("")) return;

            // loop ueber alle SQL-Anweisungen
            // String[] items = tQuelle.Text.Trim().Split(new Char[] { '\r', '\n' });
            String[] sql_commands = sSQL.Split(new Char[] { ';' });
            Time1 = DateTime.Now;
            foreach (String sql in sql_commands) {
                if (!sql.Trim().Equals("")) {
                    dtResult = dbs.selectDBS(sql);
                    if (dtResult != null) {
                        if (sql.ToLower().IndexOf("create") == -1 && sql.ToLower().IndexOf("alter") == -1 &&
                            sql.ToLower().IndexOf("drop") == -1 &&
                            sql.ToLower().IndexOf("update") == -1 && sql.ToLower().IndexOf("delete") == -1) {
                            tCount1.Text = dtResult.Rows.Count.ToString() + " ";
                            tCount2.Text = dtResult.Rows.Count.ToString() + " ";
                            datagrid.ItemsSource = dtResult.DefaultView;
                        }
                    }
                }
            }
            Time2 = DateTime.Now;
            // datagrid.IsVisible = true;

            TimeSpan duration1 = Time2 - Time1;
            tTime1.Text = duration1.ToString();
            tTime2.Text = duration1.ToString();
            if (sSQL.ToLower().IndexOf("create") >= 0 || sSQL.ToLower().IndexOf("alter") >= 0 ||
                sSQL.ToLower().IndexOf("drop") >= 0 ||
                sSQL.ToLower().IndexOf("update") >= 0 || sSQL.ToLower().IndexOf("delete") >= 0) {
                listBox2.Items.Clear();
                setTables("");
            }

            Ribbon.SelectedIndex = 2;
        }



        private void clearFontMenues(object sender) {
            bnFont8.IsChecked = false;
            bnFont10.IsChecked = false;
            bnFont12.IsChecked = false;
            bnFont14.IsChecked = false;
            bnFont18.IsChecked = false;
            bnFont20.IsChecked = false;
            bnFont24.IsChecked = false;
            bnFont28.IsChecked = false;
            RibbonCheckBox bnItem = (RibbonCheckBox)sender;
            bnItem.IsChecked = true;
        }

        private void bnFont_Click(object sender, RoutedEventArgs e) {
            RibbonCheckBox bn = (RibbonCheckBox)sender;
            clearFontMenues(sender);
            setFontsize(Convert.ToInt32(bn.Tag));

        }


        private void setFontsize(int size) {
            // this.fontsize = size;
            tInput.FontSize = size;
            listBox1.FontSize = size;
            listBox2.FontSize = size;
            datagrid.FontSize = size;
            //toolbar1.FontSize = size;
            //mainMenu.FontSize = size;
        }

        private void bnOpenSQLText_Click(object sender, RoutedEventArgs e) {
            OpenSQLText();
        }

        private void bnSaveSQLText_Click(object sender, RoutedEventArgs e) {
            SaveSQLText();
        }

        private void bnCut_Click(object sender, RoutedEventArgs e) {
            tInput.Cut();
        }

        private void bnSelectAll_Click(object sender, RoutedEventArgs e) {
            tInput.SelectAll();
        }
        private void bnCopy_Click(object sender, RoutedEventArgs e) {
            tInput.Copy();
        }

        private void bnPaste_Click(object sender, RoutedEventArgs e) {
            tInput.Paste();
        }

        private void bnUndo_Click(object sender, RoutedEventArgs e) {
            tInput.Undo();
        }

        private void bnRedo_Click(object sender, RoutedEventArgs e) {
            tInput.Redo();
        }

        private void bnStart_Click(object sender, RoutedEventArgs e) {
            Start();
        }

        private void bnExportClipboard_Click(object sender, RoutedEventArgs e) {
            StringBuilder dbText = new StringBuilder();
            DataRowCollection drc = dtResult.Rows;
            DataColumnCollection columns = dtResult.Columns;
            // foreach (DataColumn column in columns) {
            for (int i = 0; i < columns.Count; i++) {
                if (i < (columns.Count - 1)) {
                    dbText.Append(columns[i].ColumnName);
                    dbText.Append("\t");
                }
                else
                    dbText.Append(columns[i].ColumnName);
            }
            dbText.AppendLine();
            foreach (System.Data.DataRow row in dtResult.Rows) {
                Object[] objs = row.ItemArray;
                for (int i = 0; i < objs.Length; i++) {
                    Object obj = objs[i];
                    if (i < (objs.Length - 1)) {
                        dbText.Append(obj.ToString());
                        dbText.Append("\t");
                    }
                    else
                        dbText.Append(obj.ToString());
                }
                dbText.AppendLine();
            }
            Clipboard.SetText(dbText.ToString());
            MessageBox.Show("Die Ergebnisse sind in der Zwischenablage gespeichert", "Hinweis",
                MessageBoxButton.OK, MessageBoxImage.Information);
        }

        private void bnExportComExcel_Click(object sender, RoutedEventArgs e) {
            //var xl = new Excel.Application();
            //var wb = xl.Workbooks.Add();


            //DataColumnCollection cols = dtResult.Columns;
            //int anzCols = cols.Count;
            //for (int i = 0; i < anzCols; i++) {
            //    xl.Cells[1, i + 1] = cols[i].Caption;
            //    xl.Cells[1, i + 1].Select();
            //    xl.ActiveCell.Font.Size = 12;
            //    xl.ActiveCell.Font.Bold = true;
            //}

            //String title = Title;
            //var ws = (Microsoft.Office.Interop.Excel.Worksheet)wb.ActiveSheet;
            //int MAX = 10;
            //Object[,] items = new Object[MAX, anzCols];

            //int r = 1;
            //int n = 0;
            //int iIndex = 0;
            //int row = 0;
            //int rowOffset = 0;
            //foreach (System.Data.DataRow dbRow in dtResult.Rows) {
            //    r++;
            //    Title = iIndex + "  /  " + dtResult.Rows.Count;
            //    Object[] objs = dbRow.ItemArray;
            //    for (int i = 0; i < objs.Length; i++) {
            //        items[n, i] = objs[i];
            //    }
            //    n++;
            //    rowOffset++;
            //    if (n == MAX) {
            //        ws.Range["A2"].Offset[row].Resize[MAX, anzCols].Value = items;
            //        n = 0;
            //        rowOffset = 0;
            //        row += MAX;
            //    }
            //}
            //if (n > 0) {
            //    ws.Range["A2"].Offset[row].Resize[rowOffset, anzCols].Value = items;
            //}


            //xl.Visible = true;
            //Title = title;
        }


        private void bnExportExcel_Click(object sender, RoutedEventArgs e) {
            SaveFileDialog saveFileDialog1 = new SaveFileDialog();
            saveFileDialog1.Filter = "Excel-Datei (*.xls)|*.xls|All files (*.*)|*.*";
            saveFileDialog1.FilterIndex = 1;// zählt von 1 !
            saveFileDialog1.DefaultExt = ".xls";
            saveFileDialog1.InitialDirectory = Environment.CurrentDirectory;
            if (saveFileDialog1.ShowDialog() == true) {
                string sFilename = saveFileDialog1.FileName.ToString();
                Export2EXCEL exportexcel = new Export2EXCEL(sFilename);
                exportexcel.openFile();
                int r = 0;
                foreach (System.Data.DataRow row in dtResult.Rows) {
                    r++;
                    Object[] objs = row.ItemArray;
                    for (int i = 0; i < objs.Length; i++) {
                        if (objs[i] is String)
                            exportexcel.Write(i, r, (String)objs[i]);   // col row string
                        else if (objs[i] is int)
                            exportexcel.Write(i, r, (int)objs[i]);   // col row string
                        else if (objs[i] is double)
                            exportexcel.Write(i, r, (double)objs[i]);   // col row string
                        else if (objs[i] is DateTime) {
                            exportexcel.Write(i, r, ((DateTime)objs[i]).ToString("dd.MM.yyyy"));   // col row string
                        }
                        else if (objs[i] is Decimal) {
                            exportexcel.Write(i, r, ((Decimal)objs[i]).ToString("dd.MM.yyyy"));   // col row string
                        }
                        else if (objs[i] is bool) {
                            exportexcel.Write(i, r, ((DateTime)objs[i]).ToString("dd.MM.yyyy"));   // col row string
                        }
                        else {
                            exportexcel.Write(i, r, (String)objs[i]);   // col row string
                        }
                    }
                }
                exportexcel.closeFile();
                Process Prog = new Process();
                Prog.StartInfo.FileName = "excel.exe";
                Prog.StartInfo.Arguments = sFilename;
                Prog.Start();
            }
            //MessageBox.Show("Die Ergebnisse sind in der Zwischenablage gespeichert", "Hinweis",
            //    MessageBoxButton.OK, MessageBoxImage.Information);
        }

        private void RibbonWindow_KeyDown(object sender, KeyEventArgs e) {
            if (Keyboard.IsKeyDown(Key.F5)) {
                e.Handled = true;
                Start();
            }
            if (e.KeyboardDevice.IsKeyDown(Key.LeftCtrl) || e.KeyboardDevice.IsKeyDown(Key.RightCtrl)) {
                switch (e.Key) {
                    case Key.A:
                        bnSelectAll_Click(null, null);
                        break;
                    case Key.C:
                        bnCopy_Click(null, null);
                        break;
                    case Key.O:
                        OpenSQLText();
                        break;
                    case Key.S:
                        SaveSQLText();
                        break;
                    case Key.X:
                        bnCut_Click(null, null);
                        break;
                    case Key.Y:
                        bnRedo_Click(null, null);
                        break;
                    case Key.Z:
                        bnUndo_Click(null, null);
                        break;
                    case Key.V:
                        bnPaste_Click(null, null);
                        break;
                }
            }
        }


    }
}

