﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;


using System.Net;   // webClient
using System.Diagnostics;
using System.Windows.Threading;
using Company_Vorlesung.DBS_Klassen;
using System.Data;
using Company_Vorlesung.MyMessageBox;
using System.ComponentModel;
using System.Threading;

namespace Company_Vorlesung {
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window {

        private String dbtable = "MainWindow";  // für Windows Position !

        private const String databaseFile = "company.mdb";

        private byte bg_red = 255;
        private byte bg_green = 255;
        private byte bg_blue = 255;

            


        private DispatcherTimer dispatcherTimer = new DispatcherTimer();
        private bool mustCloseUpdate = false;

        public MainWindow() {
            InitializeComponent();
        }

        private void Window_Loaded(Object sender, RoutedEventArgs e) {
            Basis.WriteRegString(InstallRegistry.C_REG_ADR, "Programmdirectory", Basis.getProgPath());

            if (checkUpdate()) {
                // 
                mustCloseUpdate = true;   // dispatcherTimer_Tick
                // Close();
            }
            else {
                mustCloseUpdate = false;
            }

            // wenn ein Update, muss der WebInstaller aufgerufen werden ==>  funktioniert
            // Es muss aber dann das Programm sofort beendet werden
            // da gab es Probleme, deshalb der Timer
            dispatcherTimer.Tick += new EventHandler(dispatcherTimer_Tick);
            dispatcherTimer.Interval = TimeSpan.FromMilliseconds(500);
            Basis.setWindowPositionFromRegistry(this, InstallRegistry.C_REG_ADR + "\\position", "table" + dbtable);


            datagrid.IsReadOnly = true;
            datagrid.CanUserAddRows = false;
            datagrid.CanUserDeleteRows = false;
            datagrid.CanUserResizeColumns = true;
            datagrid.CanUserSortColumns = true;

            this.Background = new SolidColorBrush(Color.FromRgb(bg_red, bg_green, bg_blue));

        }


        private void dispatcherTimer_Tick(object sender, EventArgs e) {
            // kiesendahl
            CommandManager.InvalidateRequerySuggested();
            if (mustCloseUpdate) {
                Close();
            }
        }

        private bool checkUpdate() {
            try {
                String url = "http://www.miwilhelm.de/version-mw-2018.bin";
                byte b1 = (byte)Basis.ReadRegInt(InstallRegistry.C_REG_ADR, "Version1");
                byte b2 = (byte)Basis.ReadRegInt(InstallRegistry.C_REG_ADR, "Version2");
                byte b3 = (byte)Basis.ReadRegInt(InstallRegistry.C_REG_ADR, "Version3");

                // gibt es schon einen Datumseintrag?
                bool canCheckUpdate = Basis.ReadRegBool(InstallRegistry.C_REG_ADR, "canCheckUpdate");

                if (!canCheckUpdate) {
                    Basis.WriteRegInt(InstallRegistry.C_REG_ADR, "Day", 1); // heute schon update?
                    Basis.WriteRegInt(InstallRegistry.C_REG_ADR, "Month", 1);
                    Basis.WriteRegInt(InstallRegistry.C_REG_ADR, "Year", 2018);
                    Basis.WriteRegInt(InstallRegistry.C_REG_ADR, "Version1", 18);  // 2018
                    Basis.WriteRegInt(InstallRegistry.C_REG_ADR, "Version2", 4);   // monat
                    Basis.WriteRegInt(InstallRegistry.C_REG_ADR, "Version3", 26);
                    Basis.WriteRegBool(InstallRegistry.C_REG_ADR, "canCheckUpdate", true);
                    canCheckUpdate = true;
                }
                if (canCheckUpdate) {
                    // es wird nur täglich "updatet"
                    int day = System.DateTime.Now.Day;
                    int month = System.DateTime.Now.Month;
                    int year = System.DateTime.Now.Year;
                    int dayReg = Basis.ReadRegInt(InstallRegistry.C_REG_ADR, "Day");
                    int monthReg = Basis.ReadRegInt(InstallRegistry.C_REG_ADR, "Month");
                    int yearReg = Basis.ReadRegInt(InstallRegistry.C_REG_ADR, "Year");
                    if (day != dayReg || month != monthReg || year != yearReg) {
                        try {
                            using (var client = new WebClient()) {   //using System.Net;
                                byte[] webbytes = client.DownloadData(url);  // 4 bytes
                                StringBuilder sb = new StringBuilder();
                                foreach (byte b in webbytes) {
                                    sb.AppendLine(b.ToString());
                                }
                                int versionRegistry = ((2000 + b1) * 10000 + b2) * 100 + b3;
                                int versionWeb = ((2000 + webbytes[0]) * 10000 + webbytes[1]) * 100 + webbytes[2];
                                sb.AppendLine("versionReg: " + versionRegistry);
                                sb.AppendLine("versionWeb: " + versionWeb);

                                if (versionRegistry < versionWeb) {
                                    WUpdateReadYesNo dialogUpdate = new WUpdateReadYesNo(this, (2000 + b1).ToString() + "." + b2 + "." + b3, (2000 + webbytes[0]).ToString() + "." + webbytes[1] + "." + webbytes[2]);
                                    dialogUpdate.ShowDialog();
                                    if (dialogUpdate.retcode) {
                                        String companypath = Basis.getProgPath();  // mit \\
                                        String webinstall = companypath + "update\\Install_WebInstaller.exe";

                                        // das web-Install-Programm muss installiert werden
                                        // danach schließt das aktuelle Programm
                                        Process Prog = new Process();
                                        Prog.StartInfo.Arguments = "";
                                        Prog.StartInfo.FileName = webinstall;
                                        Prog.Start();
                                        Prog.Close();
                                        return true;  // Close

                                    }
                                }  // if (versionRegistry < versionWeb) {
                            }  // using (var client = new WebClient()) {   //using System.Net;
                        }
                        catch (Exception e) {
                            Basis.Message("Kein Internet verfügbar\r\n" + e.ToString(), "Hinweis");
                        }
                    }
                }

            }
            catch (Exception e) {
                Basis.Message("Fehler beim Update-Check aus dem Internet\r\n" + e.ToString(), "Hinweis");
            }
            return false;  // kein Close
        }  // checkUpdate


        private void bnOpenDatabase_Click(Object sender, RoutedEventArgs e) {
            openDataBase();
        }


        private void openDataBase() {
            Dbs dbs = Dbs.getDbsInstance();
            dbs.setDbs(Basis.getProgPath() + databaseFile);

            string sql = "SELECT * FROM employee";
            DataTable dtResult = dbs.selectDBS(sql);
            if (dtResult != null) {
                datagrid.ItemsSource = dtResult.DefaultView;
            }
            // hier wird die Datenbank geöffnet
            loadUpdates();
        }  // openDataBase


        // liest aus der geöffneten Datenbank die version.
        // Diese sollte DA SEIN !
        public int readDbVersion() {
            try {
                Dbs dbs = Dbs.getDbsInstance();
                String sql = "select dbVersion FROM " + "DBVERSION " + ";";
                DataTable dt = dbs.selectDBS(sql);
                foreach (System.Data.DataRow row in dt.Rows) {
                    return (int)row.ItemArray[0];
                }
            }
            catch (Exception) {
                Basis.ErrorMsg("Fehlerhafte dbVersion", "Fehler");
            }
            return 1;
        }

        private void loadUpdates() {
            try {
                InstallRegistry.DbVersion = readDbVersion();
                if (InstallRegistry.DbVersion < UpdateSQL.AnzSQL) {
                    WLoadUpdate dialog = new WLoadUpdate(this);
                    dialog.ShowDialog();
                }
            }
            catch (Exception) {

            }
        }  //   loadUpdates


        private void bnSetWebVersion_Click(Object sender, RoutedEventArgs e) {
            (new WSetWebVersion()).ShowDialog();
        }

        private void bnWebInstaller_Click(Object sender, RoutedEventArgs e) {
            (new WWebInstaller()).ShowDialog();
        }

        private void bnWebConsole_Click(Object sender, RoutedEventArgs e) {
            Dbs dbs = Dbs.getDbsInstance();
            dbs.setDbs(Basis.getProgPath() + databaseFile);
            (new WDBConsole(this)).ShowDialog();
        }

        private void bnColorDialog_Click(Object sender, RoutedEventArgs e) {
            WColorDialog dialog = new WColorDialog(this);
            dialog.setColor(bg_red, bg_green, bg_blue);
            dialog.ShowDialog();
            if (dialog.Retcode) {
                bg_red = dialog.Red;
                bg_green = dialog.Green;
                bg_blue = dialog.Blue;
                this.Background = new SolidColorBrush(Color.FromRgb(bg_red, bg_green, bg_blue));
            }

        }

        private void bnMessageBox_Click(Object sender, RoutedEventArgs e) {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("Meier, Susanne");
            sb.AppendLine("Schulz, Ingo");
            sb.AppendLine("Toralf, Hugo");
            WMessageBoxEditor dialog = new WMessageBoxEditor(this, "Import der Noten aus der Zwischenablage", "Folgende Noten waren schon vorhanden", sb.ToString());
            dialog.ShowDialog();
        }

        private void bnYesNoDialog_Click(Object sender, RoutedEventArgs e) {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("Meier, Chantal");
            sb.AppendLine("Neumann, Anton");
            sb.AppendLine("Neufeld, Hans");
            WMessageYesNoBoxEditor dialog = new WMessageYesNoBoxEditor(this, "Löschen der Noten", "Sollen die Noten folgender Studierender gelöscht werden?", sb.ToString());
            dialog.ShowDialog();
            if (dialog.RetcodeYes) {
            }
        }



        private void Window_Closing(Object sender, System.ComponentModel.CancelEventArgs e) {
            e.Cancel = false;
            Basis.writeWindowPosition2Registry(this, InstallRegistry.C_REG_ADR + "\\position", "table" + dbtable);
        }

        private BackgroundWorker worker = null;
        private void bnBackgroundWorker_Click(Object sender, RoutedEventArgs e) {
            progressbar.Visibility = Visibility.Visible;

            progressbar.Maximum = 12;  // 
            progressbar.Value = 0;
            progressbar.Visibility = System.Windows.Visibility.Visible;

            worker = new BackgroundWorker();
            worker.WorkerReportsProgress = true;
            worker.DoWork += runWorker;
            worker.ProgressChanged += worker_ProgressChanged;
            worker.RunWorkerAsync();
        }


        void worker_ProgressChanged(object sender, ProgressChangedEventArgs e) {
            progressbar.Value = e.ProgressPercentage;
            if (e.ProgressPercentage == progressbar.Maximum) {
                progressbar.Value = 0;
                progressbar.Visibility = System.Windows.Visibility.Hidden;
                Basis.Message("Die Noten wurden erfolgreich gelöscht", "Hinweis");
            }
        }

        private void runWorker(object sender, DoWorkEventArgs e) {
            int progressbarValue = 0;
            Random random = new Random(System.DateTime.Now.Second);

            Thread.Sleep( (int) (random.NextDouble()*3000) );
            worker.ReportProgress(++progressbarValue);
            Thread.Sleep(10);

            Thread.Sleep((int)(random.NextDouble() * 3000));
            worker.ReportProgress(++progressbarValue);
            Thread.Sleep(10);

            Thread.Sleep((int)(random.NextDouble() * 3000));
            worker.ReportProgress(++progressbarValue);
            Thread.Sleep(10);

            Thread.Sleep((int)(random.NextDouble() * 3000));
            worker.ReportProgress(++progressbarValue);
            Thread.Sleep(10);

            Thread.Sleep((int)(random.NextDouble() * 3000));
            worker.ReportProgress(++progressbarValue);
            Thread.Sleep(10);

            Thread.Sleep((int)(random.NextDouble() * 3000));
            worker.ReportProgress(++progressbarValue);
            Thread.Sleep(10);

            Thread.Sleep((int)(random.NextDouble() * 3000));
            worker.ReportProgress(++progressbarValue);
            Thread.Sleep(10);

            Thread.Sleep((int)(random.NextDouble() * 3000));
            worker.ReportProgress(++progressbarValue);
            Thread.Sleep(10);

            Thread.Sleep((int)(random.NextDouble() * 3000));
            worker.ReportProgress(++progressbarValue);
            Thread.Sleep(10);

            Thread.Sleep((int)(random.NextDouble() * 3000));
            worker.ReportProgress(++progressbarValue);
            Thread.Sleep(10);

            Thread.Sleep((int)(random.NextDouble() * 3000));
            worker.ReportProgress(++progressbarValue);
            Thread.Sleep(10);

            Thread.Sleep((int)(random.NextDouble() * 3000));
            worker.ReportProgress(++progressbarValue);
            Thread.Sleep(10);

        }



    }
}
